package jtrek.rmi;

import java.rmi.*;
import java.rmi.activation.*;
import java.net.*;
import java.io.*;

public class ObstrekServer extends Activatable implements ObstrekServerInterface {
    private Deactivator deactivator = null;
    Socket tcp_socket = null;
    DatagramSocket udp_socket = null;
    OutputStream out = null;
    InputStream in = null;
    ServerSocket server_socket = null;

    public ObstrekServer(ActivationID id, MarshalledObject data) throws RemoteException {
	super(id, 0);
    }

    public DatagramPacket udp_socket_receive(DatagramPacket packet) throws RemoteException {
	System.out.println("udp_socket_receive");
	try { udp_socket.receive(packet); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
	return packet;
    }

    public boolean udp_socket_is_null() throws RemoteException {
	System.out.println("udp_socket_is_null");
	return (udp_socket == null);
    }

    public void udp_socket_close() throws RemoteException {
	System.out.println("udp_socket_close");
	udp_socket.close();
    }

    public void udp_socket_set_null() throws RemoteException {
	System.out.println("udp_socket_set_null");
	udp_socket = null;
    }

    public void udp_socket_send(DatagramPacket udp_packet) throws RemoteException {
	System.out.println("udp_socket_send");
	try { udp_socket.send(udp_packet); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void udp_socket_new(int udp_local_port) throws RemoteException {
	System.out.println("udp_socket_new");
	try { udp_socket = new DatagramSocket(udp_local_port); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void udp_socket_setSoTimeout(int timeout) throws RemoteException {
	System.out.println("udp_socket_setSoTimeout");
	try { udp_socket.setSoTimeout(timeout); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public InetAddress udp_socket_getLocalAddress() throws RemoteException {
	System.out.println("udp_socket_getLocalAddress");
	return udp_socket.getLocalAddress();
    }

    public int udp_socket_getLocalPort() throws RemoteException {
	System.out.println("udp_socket_getLocalPort");
	return udp_socket.getLocalPort();
    }

    public byte[] tcp_socket_in_read(byte[] b, int off, int len) throws RemoteException {
	System.out.println("tcp_socket_in_read");
	try { 
	    int i = in.read(b,off,len); 
	    System.out.println("tcp_socket_in_read: " + i);
	    for (int j=0; j < i; j++)
		System.out.print(" "+Integer.toHexString(b[j]));
	    System.out.println("");

	    byte[] b1 = new byte[i];
	    System.arraycopy(b,off,b1,0,i);
	    return b1;
	} catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void tcp_socket_new(InetAddress host, int port) throws RemoteException {
	System.out.println("tcp_socket_new");
	try { tcp_socket = new Socket(host,port); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
	System.out.println(tcp_socket);
    }

    public void tcp_socket_getOutputStream() throws RemoteException {
	System.out.println("tcp_socket_getOutputStream");
	try { out = tcp_socket.getOutputStream(); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void tcp_socket_getInputStream() throws RemoteException {
	System.out.println("tcp_socket_getInputStream");
	try { in = tcp_socket.getInputStream(); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public boolean tcp_socket_is_null() throws RemoteException {
	System.out.println("tcp_socket_is_null");
	return (tcp_socket == null);
    }

    public void tcp_socket_close() throws RemoteException {
	System.out.println("tcp_socket_close");
	try { tcp_socket.close(); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void tcp_socket_set_null() throws RemoteException {
	System.out.println("tcp_socket_set_null");
	tcp_socket = null;
    }

    public InetAddress tcp_socket_getInetAddress() throws RemoteException {
	System.out.println("tcp_socket_getInetAddress");
	return tcp_socket.getInetAddress();
    }

    public int tcp_socket_getPort() throws RemoteException {
	System.out.println("tcp_socket_getPort");
	return tcp_socket.getPort();
    }

    public void tcp_socket_out_write(byte[] b, int off, int len) throws RemoteException {
	System.out.println("tcp_socket_out_write");
	try { out.write(b, off, len); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public boolean tcp_socket_out_is_null() throws RemoteException {
	System.out.println("tcp_socket_out_is_null");
	return (out == null);
    }

    public int tcp_reader_in_available() throws RemoteException {
	System.out.println("tcp_reader_in_available");
	try { 
	    int i = in.available(); 
	    System.out.println("tcp_reader_in_available: " + i);
	    return i;
	} catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void server_socket_new(int port, int backlog) throws RemoteException {
	System.out.println("server_socket_new");
	 try {server_socket = new ServerSocket(port,backlog); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void server_socket_setSoTimeout(int timeout) throws RemoteException {
	System.out.println("server_socket_setSoTimeout");
	try { server_socket.setSoTimeout(timeout); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void assign_tcp_socket_server_socket_accept() throws RemoteException {
	System.out.println("assign_tcp_socket_server_socket_accept");
	try { tcp_socket = server_socket.accept(); } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public boolean server_socket_is_null() throws RemoteException {
	System.out.println("server_socket_is_null");
	return (server_socket == null);
    }

    public void server_socket_close() throws RemoteException {
	System.out.println("server_socket_close");
	try { server_socket.close();  } catch (Exception e) { throw new RemoteException(e.getMessage(),e); }
    }

    public void deactivate() throws RemoteException {
	System.out.println("deactivate");
	deactivator = new Deactivator( this, getID() );
	deactivator.start();
    }
}
